/***************************************************************************************************
FILE: DDS Comb.c
HISTORY: Started 2014 September, Alan Stummer, astummer-at-physics.utoronto.ca
COMPILER/IDE:  Dynamic C 10.72
PLATFORM:  RCM6760
PROJECT OVERVIEW FROM ELECTRICAL PERSPECTIVE:
	Using the AD9959 quad DDS, four almost independant DDS cores generate four RF signals from 10MHZ
	to 175MHz (500MSPS, Nyquist 250MHz) with adjustable frequencies, amplitudes and phases, plus
	sweeps of the frequency.
PERIPHERALS:
  - just the AD9959 quad DDS
RABBIT'S ROLE IN THE PROJECT:
The Rabbit is between the host computer and the DDS.  The Rabbit:
  - gets an IP address via DHCP then connects to a host
  - receives operation commands and parameters from the host with UDP
BROADCAST ID DETAILS:
   When a unit get its IP address, either static or DHCP, it broadcasts the unit's name, a type
   identifier and IP address. The Type identifier for DDS Comb is "C".
***************************************************************************************************/

//Project specific constants
//#define DO_DEBUG  	  //normally commented out, otherwise will allow debugging through serial port.
#define UNIT_NAME		"DDS Comb #1"																	 //max 20 chars.
#define VERSION		"1.0.1"													  //string appears on the display.
#define TYPE_ID		"C"									 //when broadcasting for a host, gives unit type.
#define HB_TIMEOUT	2500				//max time between incoming heartbeats before loss of host (mS).
#define HOST_PORT		37829										  //always this port for my projects. Because.
#define F_MIN        30000																	//minimum frequency (Hz).
#define F_MAX        175000000															//maximum frequency (Hz).
#define FREQ2FTW		8.589934592													  //2^32 / 500MHz scale factor.

//Project files
#include "AD9959 Registers.h"								 //register names for the AD9958/9 dual/quad DDS.
#use "DDS Comb Hardware.c"									  			  //project specific hardware functions.
#use "DDS Comb Commands.c"													//project specific command functions.
#use "Display NHD-0220D3Z.c"					//Newhaven Displays NHD-0220D3Z-xxx-xxx display functions.

//Dynamic C directives and required defines
#class auto
#define CDRIVE_TXD       3										  				//serial port C must use PC3 only.
#define COUTBUFSIZE    127																  //serial port C Tx buffer.
#define CINBUFSIZE     127														  		  //serial port C Rx buffer.
#define STDIO_DISABLE_FLOATS												//disable floating point for std I/O.
#define TCPCONFIG 5					  															//1=static IP, 5=DHCP.
#define MAX_UDP_SOCKET_BUFFERS 1					//max number of UDP sockets that can be opened at once.
#define UDP_BUF_SIZE  4095
#define DINBUFSIZE    1023
#define DOUTBUFSIZE   1023
//#define NET_DEBUG				 				  //enable this line to debug network such as sock_init().

//Dynamic C library fies
#use "dcrtcp.lib"																			 //Rabbit's TCP/UDP library.

//Precomiler directives
#ifndef DO_DEBUG
	#nodebug								// disables insertion of breakpoints for debugging, this file only.
#endif

/*** BeginHeader Pause*/
//NB: Must use this BeginHeader and EndHeader exactly with first function name in header as shown.
//Possibly globals can be outside of this header, not sure.
nodebug void Pause (int mS);
//Global variables
udp_Socket HostSock;
int  Amplitude[4], RampUD[4];										 //amplitude and amplitude ramping up/down.
char CfrShadow[4][4];
char MyIPstr[16], Buf[100];								 //global and dangerous buffer.  Use at own risk.
/*** EndHeader */


/************************************** main() ****************************************************/
void main()  {
	int Bytes, ConnectedToHost;
	unsigned long LastIncomingHb, LastBroadcast;
	char IdBroadcast[40], Cmd[1550], Buf2[100];		 //Cmd is incoming packet and as it is processed.
	longword MyIPhex;			//for retrieving IP address.

	RabbitBoot();																								  //DAS BOOT.
	DisplayBoot();
	DdsBoot();
	DisplayWrite (UNIT_NAME, "Welcome!", BRIGHT);
	Pause (1000);

	sprintf (Buf,  "Rabbit: %s", VERSION);											  //show the Rabbit version.
	#ifdef DO_DEBUG
		strcat (Buf, "dbg");
	#endif
	DisplayWrite (UNIT_NAME, Buf, BRIGHT);
	Pause (2000);

	DisplayWrite (UNIT_NAME, "Starting phy...", BRIGHT);									//START PHY DEVICE.
	Pause (1000);
	sock_init();
	DisplayWrite (UNIT_NAME, "Waiting for cable...", BRIGHT);
	Pause (1000);
	while (ifpending(IF_DEFAULT) == IF_COMING_UP)  tcp_tick(NULL);
	//next line, third param: 0=first to connect is only allowed, 1=any can connect.
	if (!udp_open(&HostSock, HOST_PORT, 0, HOST_PORT, NULL));					  //OPEN UDP HOST SOCKET.
  	else  {
		while(!sock_established(&HostSock))
  			if (!tcp_tick(&HostSock))  break;
  	}

	DisplayWrite ("Wait for IP address", "from DHCP server...", BRIGHT);				 //GET AND SHOW IP.
	MyIPhex = 0;
	while (!MyIPhex)  {					//if static IP will have it right away, if DHCP wait for server.
		tcp_tick (NULL);
		ifconfig(IF_ETH0, IFG_IPADDR, &MyIPhex, IFS_END);
		Pause (500);
	}
	inet_ntoa (MyIPstr, MyIPhex);												 //convert hex to friendly string.
	sprintf (Buf2, "UDP Port %u", HOST_PORT);
	DisplayWrite (MyIPstr, Buf2, BRIGHT);											  //display the IP and port.
	sprintf (IdBroadcast, "I%s%s                   ", TYPE_ID, UNIT_NAME);		//string to broadcast.
	IdBroadcast[22] = 0x00;								  //truncate after unit name, total 22 bytes so far.
	strcat (IdBroadcast, MyIPstr);														//this destroys MyIPstr!!
	strcat (IdBroadcast, "         ");
	IdBroadcast[37] = 0x00;							  //truncate after 37 bytes to allow "xxx.xxx.xxx.xxx".
	Pause (2000);

	DisplayWrite (UNIT_NAME, "Waiting for host...", DIM);										 //HOUSEKEEPING.
	#ifdef DO_DEBUG
	   #warnt "FINAL RELEASE WARNING: Running with debugging turned on!"
	   printf ("My IP is %s\n", MyIPstr);
	#endif
	LastIncomingHb  = 0;													 //never recieved a heartbeat from host.
	ConnectedToHost = 0;																 //are not connected to a host.
	for (Bytes = 0; Bytes < 4; Bytes++)  {
		RampUD[Bytes]    = 0;
		Amplitude[Bytes] = 0;
	}
	Pause (1000);


		 	/***************************** Main Loop **********************************/
	while(1) {
		if (!ConnectedToHost)  {											//BROADCAST IP ADDRESS AND UNIT NAME.
			if (LastBroadcast != SEC_TIMER)  {
				if (HostSock.hisaddr == 0)  {								  //no host yet, broadcast on subnet.
	            HostSock.hisaddr = (MyIPhex & 0xFFFFFF00) + 0xFF;
	            udp_send(&HostSock, IdBroadcast, strlen(IdBroadcast));
	            HostSock.hisaddr = 0;
	         }
	         else  udp_send(&HostSock, IdBroadcast, strlen(IdBroadcast));			  //resend to host.
				LastBroadcast = SEC_TIMER;
			}
		}

		Bytes = udp_recv (&HostSock, Cmd, 1550);									 //HANDLE INCOMING COMMANDS.
		if (Bytes > 0)  {
			Cmd[Bytes] = 0x00;
			CommandHandler (Cmd);													  //process commands from host.
			LastIncomingHb = MS_TIMER;	 									 //time of last command or heartbeat.
		}

		if (LastIncomingHb + HB_TIMEOUT > MS_TIMER)  { 	 			 //HAVE HEARTBEAT, CONNECTED TO HOST.
			if (!ConnectedToHost)  {											  				//First New Heartbeat.
				ConnectedToHost = 1;
				sprintf (Buf2, "%s", inet_ntoa (Buf, HostSock.hisaddr));
		 		DisplayWrite ("Ready, host is", Buf2, BRIGHT);
			}
		}

		else  {												  									 //NO HEARTBEAT, NO HOST.
			if (ConnectedToHost)  {															//First Missed Heartbeat.
				ConnectedToHost = 0;
				sprintf (Buf2, "%s", inet_ntoa (Buf, HostSock.hisaddr));
				if (HostSock.hisaddr)  DisplayWrite ("Lost contact to", Buf, DIM);
				else                   DisplayWrite (UNIT_NAME, "Waiting for host...", DIM);
			}
		}

		tcp_tick (NULL);										//required periodic Rabbit UDP [sic] maintenance.
	}
}


/****************************************** Pause *************************************************/
void Pause (int mS)  {//Wait given mS then leave.  Help header is in SSC Hardware.c.
//nodebug void Pause (int mS)  {//Wait given mS then leave.  Help header is in SSC Hardware.c.
	//Dynamic C can't read a help description in the root file, for some dumb reason.
	unsigned long Paws;

	#ifdef DO_DEBUG													 //run display delays faster in debug mode.
//		if (mS > 999)  mS /= 3;
	#endif

	Paws = MS_TIMER;
	while (Paws + mS > MS_TIMER);
}


/**************************************************************************************************/

